/**
 *  Copyright (c) 2014, 
 *  BTC Embedded Systems AG, Oldenburg, Germany  
 *  All rights reserved
 *  --------------------------------------------------------- 
 *  $Author: $
 *  $Date:  $
 *  $Revision: $
 **/

package com.btc.ep.base.dal.tests.it;

import java.util.ArrayList;
import java.util.List;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import com.btc.ep.base.bl.Vector;
import com.btc.ep.base.bl.VectorElement;
import com.btc.ep.base.bl.impl.VectorElementImpl;
import com.btc.ep.base.bl.impl.VectorImpl;
import com.btc.ep.base.bl.repository.VectorRepository;
import com.btc.ep.base.dal.tests.it.base.BaseTest;
import com.btc.ep.base.transactions.TransactionManager;
import com.btc.ep.core.Services;

/**
 * This class tests all methods of the message repository.
 * 
 * @author Simon Kollmann
 * @see VectorRepository
 */
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class IT_VectorRepositoryTest extends BaseTest {

	private static final long STEPS = 10000;

	private static Vector vector1;

	/**
	 * initializes resources before running all tests
	 * 
	 * @throws Exception
	 *             anything
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		System.out.println("setUpBeforeClass IT_VectorRepositoryTest");

		deleteDatabase();

		int steps = (int) STEPS;
		int stepsPerCollection = 10000;

		/*
		 * vector 1
		 */

		long startTime = System.currentTimeMillis();

		vector1 = createVector("vector1", steps, stepsPerCollection);

		System.out.println("vector 1 create time: "
				+ (System.currentTimeMillis() - startTime));

	}

	/**
	 * Persist Vector
	 */
	private static Vector persistVector(String name) {
		TransactionManager tm = Services.get(TransactionManager.class);
		tm.init(jointPointId, false);

		VectorRepository vectorRepository = Services
				.get(VectorRepository.class);
		Vector vector = new VectorImpl();
		vector.setName("name");
		vector = vectorRepository.persistVector(vector);

		tm.commit(jointPointId);
		tm.close(jointPointId);
		return vector;
	}

	/**
	 * Persist Vector
	 */
	// @Transactional
	private static Vector createVector_OneTransaction(String name, int steps,
			int stepsPerCollection) {

		TransactionManager tm = Services.get(TransactionManager.class);
		tm.init(jointPointId, false);

		long startTime = System.currentTimeMillis();

		Vector vector = persistVector(name);

		for (int m = 0; m < steps / stepsPerCollection; m++) {

			createColletion(vector.getUid(), m, stepsPerCollection);
		}

		System.out.println(" create Vector " + name + " complete");
		System.out.println("vector " + name + " create time: "
				+ (System.currentTimeMillis() - startTime));

		tm.commit(jointPointId);
		tm.close(jointPointId);

		return findVector(vector.getUid());
	}

	/**
	 * Persist Vector
	 */
	// @NotTransactional
	private static Vector createVector(String name, int steps,
			int stepsPerCollection) {

		long startTime = System.currentTimeMillis();

		Vector vector = persistVector(name);

		for (int m = 0; m < steps / stepsPerCollection; m++) {

			createColletion(vector.getUid(), m, stepsPerCollection);
		}

		System.out.println(" create Vector " + name
				+ " complete NotTransactional");
		System.out.println("vector " + name + " create time: "
				+ (System.currentTimeMillis() - startTime));

		return findVector(vector.getUid());
	}

	private static void createColletion(String vectorID, int collectionNr,
			int stepsPerCollection) {
		TransactionManager tm = Services.get(TransactionManager.class);
		tm.init(jointPointId, false);

		VectorRepository vectorRepository = Services
				.get(VectorRepository.class);
		Vector vector = vectorRepository.findVector(vectorID);

		List<VectorElement> vectorElements = new ArrayList<VectorElement>(
				stepsPerCollection);
		for (int i = 0; i < stepsPerCollection; i++) {
			long step = (collectionNr * stepsPerCollection) + i;

			VectorElementImpl ve = new VectorElementImpl(step,
					createFloatArray(step));
			vectorElements.add(ve);
		}
		vectorRepository.addVectorElements(vector, vectorElements);
		vector.incrementElementCount(vectorElements.size());

		// ?? whitout, it doesn't work
		vectorRepository.persistVector(vector);

		tm.commit(jointPointId);
		tm.close(jointPointId);
	}

	/**
	 * destroys resources after running all tests
	 * 
	 * @throws Exception
	 *             anything
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {

	}

	/**
	 * initializes before running a particular test
	 * 
	 * @throws Exception
	 *             anything
	 */
	@Before
	public void setUp() throws Exception {
	}

	/**
	 * destroys after running a particular test
	 * 
	 * @throws Exception
	 *             anything
	 */
	@After
	public void tearDown() throws Exception {
	}

	@Test
	@Ignore
	public void test99_setup_OneTransaction() {
		System.out.println("test0_setup_OneTransaction");

		deleteDatabase();

		createVector_OneTransaction("vectorA", (int) STEPS, 10000);
	}

	@Test
	@Ignore
	public void test99_setup_ManyTransactions() {
		System.out.println("test0_setup_ManyTransactions");

		deleteDatabase();

		createVector("vectorA", (int) STEPS, 10000);
	}

	/**
	 * test to find all VectorElementsOfOneVector
	 *
	 * @see VectorRepository#findVectorElements(Vector)
	 */
	@Test
	public void test1_findAllVectorElementsOfOneVector() {

		// Separate Transaction
		doing_test1_findAllVectorElementsOfOneVector();
	}

	/**
	 * Brief Function description
	 *
	 */
	private void doing_test1_findAllVectorElementsOfOneVector() {
		TransactionManager tm = Services.get(TransactionManager.class);
		tm.init(jointPointId, false);

		remarkTime();
		VectorRepository vectorRepository = Services
				.get(VectorRepository.class);
		Vector vector = vectorRepository.findVector(vector1.getUid());
		printTime("findVector");

		remarkTime();
		List<VectorElement> vectorElements = vectorRepository
				.findVectorElements(vector);
		printTime("test1_findVectorElements(vector)");

		Assert.assertEquals("check size of the founded VectorElements", STEPS,
				vectorElements.size());

		Assert.assertEquals("? - failed", vectorElements.get(1234)
				.getStepOffset(), (int) (vectorElements.get(1234).getValue()
				.get(0) * 1));

		tm.commit(jointPointId);
		tm.close(jointPointId);
	}

	/**
	 * test to find VectorElements
	 *
	 * @see VectorRepository#findVectorElement
	 */
	@Test
	public void test2_findVectorElement() {
		TransactionManager tm = Services.get(TransactionManager.class);
		tm.init(jointPointId, false);

		remarkTime();
		VectorRepository vectorRepository = Services
				.get(VectorRepository.class);
		Vector vector = vectorRepository.findVector(vector1.getUid());
		printTime("findVector");

		long step = 1234;

		remarkTime();
		VectorElement vectorElement = vectorRepository.findVectorElement(
				vector, step);
		printTime("test2_findVectorElement");

		Assert.assertEquals("check step", step, vectorElement.getStepOffset());
		Assert.assertEquals("check value", step, (long) (vectorElement
				.getValue().get(0) * 1));

		tm.commit(jointPointId);
		tm.close(jointPointId);

	}

	/**
	 * test to find VectorElements
	 *
	 * @see VectorRepository#findVectorElement
	 */
	@Test
	public void test3_insertVectorElement() {

		test3_insertOneVectorElement();

	}

	/**
	 * Brief Function description
	 *
	 * @param vector
	 */
	private void test3_insertOneVectorElement() {
		TransactionManager tm = Services.get(TransactionManager.class);
		tm.init(jointPointId, false);

		remarkTime();
		VectorRepository vectorRepository = Services
				.get(VectorRepository.class);
		Vector vector = vectorRepository.findVector(vector1.getUid());
		printTime("findVector");

		VectorElement ve = new VectorElementImpl(500, createFloatArray(500));

		remarkTime();
		// increment Elements after step
		vectorRepository.addVectorElement(vector, ve);

		printTime("test3_addVectorElement");

		tm.commit(jointPointId);
		tm.close(jointPointId);
	}

	/**
	 * test to find VectorElements
	 *
	 * @see VectorRepository#findVectorElement
	 */
	@Test
	public void test50_incrementAll() {
		TransactionManager tm = Services.get(TransactionManager.class);
		tm.init(jointPointId, false);

		VectorRepository vectorRepository = Services
				.get(VectorRepository.class);

		remarkTime();
		vectorRepository.incrAll();
		printTime("test99_incrementAll");

		tm.commit(jointPointId);
		tm.close(jointPointId);
	}

	/**
	 * test to find VectorElements
	 *
	 * @see VectorRepository#findVectorElement
	 */
	@Test
	public void test4_creation() {
		TransactionManager tm = Services.get(TransactionManager.class);
		tm.init(jointPointId, false);

		VectorRepository vectorRepository = Services
				.get(VectorRepository.class);

		remarkTime();

		Vector vector = new VectorImpl();
		vectorRepository.persistVector(vector);

		List<VectorElement> newVectorElements = new ArrayList<VectorElement>(
				(int) STEPS);

		// add a Vector Element
		for (int i = 0; i < 1000; i++) {
			VectorElementImpl ve = new VectorElementImpl(i, createFloatArray(i));
			newVectorElements.add(ve);
		}
		vectorRepository.addVectorElements(vector, newVectorElements);
		vector.incrementElementCount(newVectorElements.size());

		printTime("test4_creation");

		tm.commit(jointPointId);
		tm.close(jointPointId);

	}

	/**
	 * test to find VectorElements
	 *
	 * @see VectorRepository#findVectorElement
	 */
	// @Test
	public void test5_creation_single() {
		TransactionManager tm = Services.get(TransactionManager.class);
		tm.init(jointPointId, false);

		VectorRepository vectorRepository = Services
				.get(VectorRepository.class);

		remarkTime();

		Vector vector = new VectorImpl();
		vectorRepository.persistVector(vector);

		// add a Vector Element
		for (int i = 0; i < 1000; i++) {
			VectorElementImpl ve = new VectorElementImpl(i, createFloatArray(i));
			vectorRepository.addVectorElement(vector, ve);
			vector.incrementElementCount(1);
		}
		printTime("test5_creation_single");

		tm.commit(jointPointId);
		tm.close(jointPointId);

	}

	@Test
	public void test_createVector() {
		TransactionManager tm = Services.get(TransactionManager.class);
		tm.init(jointPointId, false);
		VectorRepository vectorRepository = Services
				.get(VectorRepository.class);

		remarkTime();

		Vector vector = new VectorImpl();
		vector.setName("name");

		vector = vectorRepository.persistVector(vector);
		printTime("test_createVector");

		tm.commit(jointPointId);
		tm.close(jointPointId);
	}

	@Test
	public void test_deleteVecData() {
		TransactionManager tm = Services.get(TransactionManager.class);
		tm.init(jointPointId, false);
		VectorRepository vectorRepository = Services
				.get(VectorRepository.class);

		Vector vector = new VectorImpl();
		vector.setName("name");

		vector = vectorRepository.persistVector(vector);
		remarkTime();
		vectorRepository.removeVector(vector);
		printTime("test_deleteVecData");

		tm.commit(jointPointId);
		tm.close(jointPointId);
	}

	private static List<Float> createFloatArray(float value) {
		List<Float> floats = new ArrayList<Float>(100);
		for (float i = 0; i < 100; i++) {
			floats.add(value + (i / 100));
		}
		return floats;
	}

	private static Vector findVector(String uid) {
		TransactionManager tm = Services.get(TransactionManager.class);
		tm.init(jointPointId, false);

		VectorRepository vectorRepository = Services
				.get(VectorRepository.class);
		Vector vector = vectorRepository.findVector(uid);

		tm.commit(jointPointId);
		tm.close(jointPointId);

		return vector;
	}
}
