/**
 *  Copyright (c) 2014, 
 *  BTC Embedded Systems AG, Oldenburg, Germany  
 *  All rights reserved
 *  --------------------------------------------------------- 
 *  $Author: $
 *  $Date:  $
 *  $Revision: $
 **/

package com.btc.ep.base.dal.internal;

import java.util.Collection;
import java.util.HashSet;

import com.btc.ep.base.bl.ModelElement;
import com.btc.ep.base.bl.impl.ModelElementImpl;
import com.btc.ep.base.dal.impl.BaseRepository;
import com.btc.ep.core.Services;
import com.btc.ep.dal.persist.PersistenceManager;
import com.btc.ep.dal.persist.impl.PersistenceService;

/**
 * This class is a standard repository.
 * 
 * @author Martin Manthey
 */
public class BaseRepositoryImpl implements BaseRepository<ModelElement> {

    /**
     * Injected by OSGi.
     */
    private PersistenceManager persistenceManager;

    /**
     * set PersistenceManager
     *
     * @param persistenceManager persistence manager
     */
    public void setPersistenceManager(PersistenceManager persistenceManager) {
        this.persistenceManager = persistenceManager;
    }

    /**
     * Get access to the PersistenceManager
     *
     * @return PersistenceManager
     */
    private PersistenceManager getPersistenceManager() {
        if (persistenceManager == null) {
            persistenceManager = Services.get(PersistenceManager.class);
        }
        return persistenceManager;
    }

    /**
     * Get easy access to the PersistenceService
     *
     * @return PersistenceService
     */
    private PersistenceService getPersistenceService() {
        return getPersistenceManager().getPersistenceService();
    }

    @Override
    public ModelElement find(String uid) {

//        return getPersistenceService().getModelElementOrNullByQuery(
//            "select distinct m from " + ModelElementImpl.class.getSimpleName() + " m where m.uid = ?1",
//            ModelElement.class, uid);
        
        return getPersistenceService().find(ModelElementImpl.class, uid);
    }

    /**
     * Load the repository aggregate by Uid
     *
     * @param uid The Uid of the ModelElement to load. Attention! It must be an uid of the type of aggregate.
     * @deprecated Use {@link com.btc.ep.base.dal.internal.BaseRepositoryImpl#find(String)}
     * @return The ModelElement with the given uid. If no element found it return NULL.
     */
    public ModelElement load(String uid) {
        return getPersistenceService().find(ModelElement.class, uid);
    }

    @Override
    public <E extends ModelElementImpl> Collection<E> findAll(Class<E> clazz) {
        return getPersistenceService().getModelElementsByQuery(
            "select distinct m from " + clazz.getSimpleName() + " m", clazz);
    }

    @Override
    public <T extends ModelElement> T update(T dmo) {

        T attachedDmo = getPersistenceService().update(dmo);
        
        return attachedDmo;
    }

    @Override
    public Collection<ModelElement> update(Collection<ModelElement> dmos) {

        Collection<ModelElement> attachedDMOs = new HashSet<ModelElement>();

        for (ModelElement dmo : dmos) {
            ModelElement attachedDmo = getPersistenceService().update(dmo);
            attachedDMOs.add(attachedDmo);
        }

        return attachedDMOs;
    }

    @Override
    public void remove(ModelElement dmo) {

        getPersistenceService().remove(dmo);
    }

    @Override
    public void remove(Collection<ModelElement> dmos) {

        for (ModelElement dmo : dmos) {
            getPersistenceService().remove(dmo);
        }
    }

    @Override
    public void removeImpl(Collection<? extends ModelElementImpl> dmos) {

        for (ModelElement dmo : dmos) {
            getPersistenceService().remove(dmo);
        }
    }

    @Override
    public boolean save() {
    
    //TODO remove method from API
//        getPersistenceService().commit();
        return true;
    }

    @Override
    public boolean revert() {
    
    //TODO remove method from API
        //FIXME does it works?

//        EntityTransaction transaction = getPersistenceService().getTransaction();
//
//        if (transaction != null) {
//            transaction.rollback();
//            persistenceManager.clear();
//            return true;
//        }
//
//        return false;
        return true;

    }

    @Override
    public boolean clearDatabase() {
        getPersistenceService().updateQuery("DELETE FROM Object"); // delete whole database
        return true;
    }
}
